@@ -1,4 +1,4 @@
-/* $Id: em8300_compat24.h,v 1.5 2006/04/10 22:31:14 nboullis Exp $
+/* $Id: em8300_compat24.h,v 1.6 2007/12/03 22:59:09 nboullis Exp $
  *
  * em8300_compat24.h -- compatibility layer for 2.4 and some 2.5 kernels
  * Copyright (C) 2004 Andreas Schultz <aschultz@warp10.net>
@@ -30,6 +30,15 @@
 typedef void irqreturn_t;
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
+#ifndef IRQF_DISABLED
+#define IRQF_DISABLED SA_INTERRUPT
+#endif
+#ifndef IRQF_SHARED
+#define IRQF_SHARED SA_SHIRQ
+#endif
+#endif
+
 /* i2c stuff */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,67)
 static inline void *i2c_get_clientdata(struct i2c_client *dev)
