@@ -23,8 +23,16 @@
 
 #include "adv717x.h"
 #include "bt865.h"
+#include "encoder.h"
 //#include <linux/sensors.h>
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+#define sysfs_create_link(kobj, target, name) 0
+#define sysfs_remove_link(kobj, name) do {} while (0)
+#else
+#include <linux/sysfs.h>
+#endif
+
 #define I2C_HW_B_EM8300 0xa
 
 struct private_data_s {
@@ -90,6 +98,12 @@ static int em8300_i2c_lock_client(struct i2c_client *client)
 	return 0;
 }
 
+#define EM8300_I2C_MAKE_LINK(link_name) \
+	do { \
+		if (sysfs_create_link(&em->dev->dev.kobj, &client->dev.kobj, link_name)) \
+			printk(KERN_WARNING "em8300_i2c: unable to create the %s link\n", link_name); \
+	} while (0)
+
 static int em8300_i2c_reg(struct i2c_client *client)
 {
 	struct em8300_s *em = i2c_get_adapdata(client->adapter);
@@ -106,6 +120,54 @@ static int em8300_i2c_reg(struct i2c_client *client)
 			em->encoder_type = ENCODER_ADV7170;
 		}
 		em->encoder = client;
+		EM8300_I2C_MAKE_LINK("encoder");
+		client->driver->command(client, ENCODER_CMD_ENABLEOUTPUT, (void *)0);
+		do {
+			struct getconfig_s data;
+			struct setparam_s param;
+			data.card_nr = em->card_nr;
+			if (client->driver->command(client,
+						    ENCODER_CMD_GETCONFIG,
+						    (void *) &data) != 0) {
+				printk("ENCODER_CMD_GETCONFIG failed\n");
+				break;
+			}
+			param.param = ENCODER_PARAM_COLORBARS;
+			param.modes = (uint32_t)-1;
+			param.val = data.config[4]?1:0;
+			client->driver->command(client,
+						ENCODER_CMD_SETPARAM,
+						&param);
+			param.param = ENCODER_PARAM_OUTPUTMODE;
+			param.val = data.config[5];
+			client->driver->command(client,
+						ENCODER_CMD_SETPARAM,
+						&param);
+			param.param = ENCODER_PARAM_PPORT;
+			param.modes = NTSC_MODES_MASK;
+			param.val = data.config[0]?1:0;
+			client->driver->command(client,
+						ENCODER_CMD_SETPARAM,
+						&param);
+			param.modes = PAL_MODES_MASK;
+			param.val = data.config[1]
+				? (data.config[0]?0:1)
+				: (data.config[0]?1:0);
+			client->driver->command(client,
+						ENCODER_CMD_SETPARAM,
+						&param);
+			param.param = ENCODER_PARAM_PDADJ;
+			param.modes = NTSC_MODES_MASK;
+			param.val = data.config[2];
+			client->driver->command(client,
+						ENCODER_CMD_SETPARAM,
+						&param);
+			param.modes = PAL_MODES_MASK;
+			param.val = data.config[3];
+			client->driver->command(client,
+						ENCODER_CMD_SETPARAM,
+						&param);
+		} while (0);
 		break;
 	case I2C_DRIVERID_BT865:
 		if (em8300_i2c_lock_client(client)) {
@@ -113,9 +175,10 @@ static int em8300_i2c_reg(struct i2c_client *client)
 		}
 		em->encoder_type = ENCODER_BT865;
 		em->encoder = client;
+		EM8300_I2C_MAKE_LINK("encoder");
 		break;
 	case I2C_DRIVERID_EEPROM:
-		/* do nothing */
+		EM8300_I2C_MAKE_LINK("eeprom");
 		break;
 	default:
 		printk(KERN_ERR "em8300_i2c: unknown client id\n");
@@ -132,6 +195,7 @@ static int em8300_i2c_unreg(struct i2c_client *client)
 	switch (client->driver->id) {
 	case I2C_DRIVERID_ADV717X:
 	case I2C_DRIVERID_BT865:
+		sysfs_remove_link(&em->dev->dev.kobj, "encoder");
 		em->encoder = NULL;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,54)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
@@ -141,6 +205,9 @@ static int em8300_i2c_unreg(struct i2c_client *client)
 #endif
 #endif
 		break;
+	case I2C_DRIVERID_EEPROM:
+		sysfs_remove_link(&em->dev->dev.kobj, "eeprom");
+		break;
 	}
 
 	return 0;
