@@ -1,4 +1,4 @@
-/* $Id: em8300_ioctl32.c,v 1.5 2006/04/10 22:31:14 nboullis Exp $
+/* $Id: em8300_ioctl32.c,v 1.6 2007/08/24 16:19:25 scop Exp $
  *
  * em8300_ioctl32.c -- compatibility layer for 32-bit ioctls on 64-bit kernels
  * Copyright (C) 2004 Nicolas Boullis <nboullis@debian.org>
@@ -32,6 +32,8 @@
 #include <linux/pci.h>
 #include <asm/io.h>
 #include <asm/uaccess.h>
+#include <linux/ioctl.h>
+#include <linux/fs.h>
 
 #include <linux/i2c.h>
 #include <linux/i2c-algo-bit.h>
