@@ -1,4 +1,4 @@
-/* $Id: em8300_ioctl32.h,v 1.1 2004/02/17 20:56:34 nboullis Exp $
+/* $Id: em8300_ioctl32.h,v 1.4 2006/04/10 22:31:14 nboullis Exp $
  *
  * em8300_ioctl32.h -- compatibility layer for 32-bit ioctls on 64-bit kernels
  * Copyright (C) 2004 Nicolas Boullis <nboullis@debian.org>
@@ -15,13 +15,21 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef EM8300_IOCTL32_H
 #define EM8300_IOCTL32_H
 
+#ifdef HAVE_COMPAT_IOCTL
+
+extern long em8300_compat_ioctl(struct file* filp, unsigned cmd, unsigned long arg);
+
+#else /* HAVE_COMPAT_IOCTL */
+
 extern void em8300_ioctl32_init(void);
 extern void em8300_ioctl32_exit(void);
 
+#endif /* HAVE_COMPAT_IOCTL */
+
 #endif /* EM8300_IOCTL32_H */
